﻿//-----------------------------------------------------------------------
// <copyright file="TemplateGroupRepository.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using System.Collections.Generic;
using System.Data.EntityClient;
using System.Linq;
using FastNET.CleanAct.Client.ManagerEngineData.Entities;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.ManagementEngine;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.ManagementEngine;

#endregion

namespace FastNET.CleanAct.Client.ManagerEngineData.Repositories
{
    /// <summary>
    /// General repository for managing TemplateGroups
    /// </summary>
    internal class TemplateGroupRepository
        : ITemplateGroupRepository
    {
        #region CTor/DTor

        /// <summary>
        /// Initializes a new instance of the TemplateGroupRepository class
        /// </summary>
        /// <param name="connection">Reference to the common EntityConnection for the repository</param>
        internal TemplateGroupRepository(EntityConnection connection)
        {
            Connection = connection;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a reference to the common EntityConnection for the repository
        /// </summary>
        protected EntityConnection Connection { get; private set; }

        #endregion

        #region IRepository<ITemplateGroupData> Members

        /// <summary>
        /// Retrieves all instances of ILogEntryData from the repository
        /// </summary>
        /// <returns>IEnumerable of the requested IData</returns>
        public IList<ITemplateGroupData> Retrieve()
        {
            List<ITemplateGroupData> returnValues = new List<ITemplateGroupData>();

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                IEnumerable<TemplateGroup> query = from item in dataSource.TemplateGroups select item;

                foreach (TemplateGroup item in query)
                {
                    returnValues.Add(item);
                    dataSource.Detach(item);
                }
            }

            return returnValues;
        }

        /// <summary>
        /// Deletes the requested ITemplateGroupData
        /// </summary>
        /// <param name="Data">ITemplateGroupData to delete from the repository</param>
        /// <exception cref="ArgumentNullException">Thrown if data is null</exception>
        /// <exception cref="InvalidCastException">Thrown if the data is of the wrong type for this datasource</exception>
        /// <exception cref="InvalidOperationException">Thrown if the ITemplateGroupData is new or already deleted</exception>
        public void Delete(ITemplateGroupData Data)
        {
            if (Data == null)
            {
                throw new ArgumentNullException("Data", "Must have a valid Data to delete");
            }

            if (!(Data is TemplateGroup))
            {
                throw new InvalidCastException("Data is not of type TemplateGroup. Cannot delete this object through this repository");
            }

            if (Data.IsNew)
            {
                throw new InvalidOperationException("Data cannot be new before deleting. There is nothing to delete.");
            }

            if (Data.IsDeleted)
            {
                throw new InvalidOperationException("Data already deleted.");
            }

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                dataSource.Attach((TemplateGroup)Data);
                dataSource.DeleteObject(Data);
                dataSource.SaveChanges();
            }
        }

        /// <summary>
        /// Saves the requested ITemplateGroupData
        /// </summary>
        /// <param name="Data">ITemplateGroupData to save to the repository</param>
        /// <exception cref="ArgumentNullException">Thrown if data is null</exception>
        /// <exception cref="InvalidCastException">Thrown if the data is of the wrong type for this datasource</exception>
        public void Save(ITemplateGroupData Data)
        {
            if (Data == null)
            {
                throw new ArgumentNullException("Data", "Must have a valid Data to delete");
            }

            if (!(Data is TemplateGroup))
            {
                throw new InvalidCastException("Data is not of type TemplateGroup. Cannot save this object through this repository");
            }

            if (!Data.IsDirty)
            {
                return;
            }

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                if (Data.IsNew)
                {
                    dataSource.AddToTemplateGroups((TemplateGroup)Data);
                    dataSource.SaveChanges();
                    dataSource.Detach(Data);
                }
                else
                {
                    dataSource.ApplyPropertyChanges("TemplateGroups", Data);
                    dataSource.SaveChanges();
                }
            }
        }

        /// <summary>
        /// Creates a new instance of ITemplateGroupData
        /// </summary>
        /// <returns>New Instance of ITemplateGroupData</returns>
        public ITemplateGroupData Create()
        {
            return new TemplateGroup();
        }

        #endregion

        #region Implementation of ITemplateGroupRepository

        /// <summary>
        /// Retrieves all template group mappings by parent template ID
        /// </summary>
        /// <param name="parentID">Parent Template to retrieve associated classes for</param>
        /// <returns>List of ITemplateGroupData with all matching results</returns>
        public IList<ITemplateGroupData> Retrieve(Identity parentID)
        {
            if (!parentID.CanFitInGuid())
            {
                throw new InvalidCastException(string.Format("Identity for this type of repository requires it to be of Guid Type. Currently is {0}.", parentID.UnderlyingType()));
            }

            // This has to be done here because LINQ does not support the inline case
            Guid searchID = parentID;

            List<ITemplateGroupData> returnValues = new List<ITemplateGroupData>();

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                IEnumerable<TemplateGroup> query = from
                                                            item
                                                            in
                                                       dataSource.TemplateGroups
                                                   where
                                                       item.ParentTemplateID == searchID
                                                   select
                                                       item;

                foreach (TemplateGroup item in query)
                {
                    returnValues.Add(item);
                    dataSource.Detach(item);
                }

                return returnValues;
            }
        }

        /// <summary>
        /// Retrieves template group mapping by parent ID and child ID
        /// </summary>
        /// <param name="parentID">Parent Template to retrieve associated mapping by</param>
        /// <param name="childID">Child Template to retrieve associated mapping by</param>
        /// <returns>ITemplateGroupData with matching result or null if none was found</returns>
        public ITemplateGroupData Retrieve(Identity parentID, Identity childID)
        {
            if (!parentID.CanFitInGuid())
            {
                throw new InvalidCastException(string.Format("Character identity for this type of repository requires it to be of Guid Type. Currently is {0}.", parentID.UnderlyingType()));
            }

            if (!childID.CanFitInGuid())
            {
                throw new InvalidCastException(string.Format("Class identity for this type of repository requires it to be of Guid Type. Currently is {0}.", childID.UnderlyingType()));
            }

            Guid pID = parentID;
            Guid cID = childID;

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                TemplateGroup foundItem = (from
                                                             item
                                                             in
                                               dataSource.TemplateGroups
                                           where
                                               item.ParentTemplateID == pID &&
                                               item.ChildTemplateID == cID
                                           select
                                               item).FirstOrDefault();

                if (foundItem != null)
                {
                    dataSource.Detach(foundItem);
                }

                return foundItem;
            }
        }

        #endregion
    }
}