﻿//-----------------------------------------------------------------------
// <copyright file="TemplateRepository.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.EntityClient;
using System.Linq;
using FastNET.CleanAct.Client.ManagerEngineData.Entities;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.ManagementEngine;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.ManagementEngine;

#endregion

namespace FastNET.CleanAct.Client.ManagerEngineData.Repositories
{
    /// <summary>
    /// General repository for managing Templates
    /// </summary>
    internal class TemplateRepository
        : ITemplateRepository
    {
        #region CTor/DTor

        /// <summary>
        /// Initializes a new instance of the TemplateRepository class
        /// </summary>
        /// <param name="connection">Reference to the common EntityConnection for the repository</param>
        internal TemplateRepository(EntityConnection connection)
        {
            Connection = connection;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a reference to the common EntityConnection for the repository
        /// </summary>
        protected EntityConnection Connection { get; private set; }

        #endregion

        #region IRepository<ITemplateData> Members

        /// <summary>
        /// Retrieves all instances of ILogEntryData from the repository
        /// </summary>
        /// <returns>IEnumerable of the requested IData</returns>
        public IList<ITemplateData> Retrieve()
        {
            List<ITemplateData> returnValues = new List<ITemplateData>();

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                IEnumerable<Template> query = from item in dataSource.Templates select item;

                foreach (Template item in query)
                {
                    returnValues.Add(item);
                    dataSource.Detach(item);
                }
            }

            return returnValues;
        }

        /// <summary>
        /// Retrieves the requested ITemplateData by an Identity
        /// </summary>
        /// <param name="ID">Identity to retrieve the data by</param>
        /// <returns>ITemplateData instance or null if one was not found</returns>
        /// <exception cref="InvalidCastException">Thrown if the identity is of the wrong type for this datasource</exception>
        public ITemplateData Retrieve(Identity ID)
        {
            if (!ID.CanFitInGuid())
            {
                throw new InvalidCastException(string.Format("Identity for this type of repository requires it to be of Guid Type. Currently is {0}.", ID.UnderlyingType()));
            }

            // This has to be done here because LINQ does not support the inline case
            Guid searchID = ID;

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                Template foundItem = (from
                                                item
                                                in
                                          dataSource.Templates
                                      where
                                          item.TemplateID == searchID
                                      select
                                          item).FirstOrDefault();

                if (foundItem != null)
                {
                    dataSource.Detach(foundItem);
                }

                return foundItem;
            }
        }

        /// <summary>
        /// Gets a list of templates by using their associated packageID
        /// </summary>
        /// <param name="packageID">Identity of the package to retrieve by or NULL to retrieve unassocated packages</param>
        /// <returns>List of ITemplateData for all found packages</returns>
        public IList<ITemplateData> RetrieveByPackageID(Identity? packageID)
        {
            if (packageID.HasValue && !packageID.Value.CanFitInGuid())
            {
                throw new InvalidCastException(string.Format("Identity for this type of repository requires it to be of Guid Type. Currently is {0}.", packageID.Value.UnderlyingType()));
            }

            List<ITemplateData> returnValues = new List<ITemplateData>();

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                IEnumerable<Template> query;

                if (packageID.HasValue)
                {
                    Guid itemID = packageID.Value;
                    query = from item in dataSource.Templates where item.Package.PackageID == itemID select item;
                }
                else
                {
                    query = from item in dataSource.Templates where item.Package == null select item;
                }

                foreach (Template item in query)
                {
                    returnValues.Add(item);
                    dataSource.Detach(item);
                }
            }

            return returnValues;
        }

        /// <summary>
        /// Deletes the requested ITemplateData
        /// </summary>
        /// <param name="Data">ITemplateData to delete from the repository</param>
        /// <exception cref="ArgumentNullException">Thrown if data is null</exception>
        /// <exception cref="InvalidCastException">Thrown if the data is of the wrong type for this datasource</exception>
        /// <exception cref="InvalidOperationException">Thrown if the ITemplateData is new or already deleted</exception>
        public void Delete(ITemplateData Data)
        {
            if (Data == null)
            {
                throw new ArgumentNullException("Data", "Must have a valid Data to delete");
            }

            if (!(Data is Template))
            {
                throw new InvalidCastException("Data is not of type Template. Cannot delete this object through this repository");
            }

            if (Data.IsNew)
            {
                throw new InvalidOperationException("Data cannot be new before deleting. There is nothing to delete.");
            }

            if (Data.IsDeleted)
            {
                throw new InvalidOperationException("Data already deleted.");
            }

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                dataSource.Attach((Template)Data);
                foreach (TemplateGroup group in from item in dataSource.TemplateGroups where item.ParentTemplateID == ((Template)Data).TemplateID || item.ChildTemplateID == ((Template)Data).TemplateID select item)
                {
                    dataSource.DeleteObject(group);
                }
                dataSource.DeleteObject(Data);
                dataSource.SaveChanges();
            }
        }

        /// <summary>
        /// Saves the requested ITemplateData
        /// </summary>
        /// <param name="Data">ITemplateData to save to the repository</param>
        /// <exception cref="ArgumentNullException">Thrown if data is null</exception>
        /// <exception cref="InvalidCastException">Thrown if the data is of the wrong type for this datasource</exception>
        public void Save(ITemplateData Data)
        {
            if (Data == null)
            {
                throw new ArgumentNullException("Data", "Must have a valid Data to delete");
            }

            if (!(Data is Template))
            {
                throw new InvalidCastException("Data is not of type Template. Cannot save this object through this repository");
            }

            if (!Data.IsDirty)
            {
                return;
            }

            using (ManagerDataEntities dataSource = new ManagerDataEntities(Connection))
            {
                if (Data.IsNew)
                {
                    dataSource.AddToTemplates((Template)Data);
                    dataSource.SaveChanges();
                    dataSource.Detach(Data);
                }
                else
                {
                    object originalItem;

                    // Create the detached object's entity key.
                    EntityKey itemKey = dataSource.CreateEntityKey("Templates", Data);

                    // Get the original item based on the entity key from the context
                    // or from the database.
                    if (dataSource.TryGetObjectByKey(itemKey, out originalItem))
                    {
                        // Call the ApplyPropertyChanges method to apply changes
                        // from the updated item to the original version.
                        dataSource.ApplyPropertyChanges(itemKey.EntitySetName, Data);
                        ((Template)originalItem).PackageReference.EntityKey = ((Template)Data).PackageReference.EntityKey;
                        dataSource.SaveChanges();
                    }
                }
            }
        }

        /// <summary>
        /// Creates a new instance of ITemplateData
        /// </summary>
        /// <returns>New Instance of ITemplateData</returns>
        public ITemplateData Create()
        {
            return new Template();
        }

        #endregion
    }
}